/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <stdio.h>
#include "ukcprog.h"
#include <string.h>

#ifdef SOLARIS
#include <sys/mnttab.h>
#include <sys/types.h>
#include <sys/statvfs.h>
#endif
#ifdef LINUX
#include <mntent.h>
#include <sys/vfs.h>
#endif
#ifdef FREEBSD
#include <sys/param.h>
#include <sys/ucred.h>
#include <sys/mount.h>
#endif
typedef struct {
	char *device_name;
	char *mnt_point;
	long size;
	long used;
	long avail;
	long t_inodes;
	long f_inodes;
}disk_stat_t;

typedef struct system_disks_t{
	disk_stat_t *disk;
	struct system_disks_t *next_disk;
}system_disks_t;

char *get_disk_stats(){
	system_disks_t *sd=NULL;
	system_disks_t *sd_ptr=sd;
	int counter=0;
	char *xml_disk_stats;
	char *xml_disk_stats_ptr;
#ifdef SOLARIS
	struct mnttab mp;
	struct statvfs df;
	FILE *f;
#endif
#ifdef LINUX
	struct mntent *mp;
	struct statfs df;
	FILE *f;
#endif
#ifdef FREEBSD
	int nummnt;
	struct statfs *mp;
#endif

#ifdef SOLARIS
	if ((f=fopen("/etc/mnttab", "r" ))==NULL){
		errf("Failed to open mounts (%m)");
		return NULL;
	}
	while((getmntent(f, &mp)) == 0){
		if ((statvfs(mp.mnt_mountp, &df)) !=0){
			errf("Failed to gets fs stats (%m)");
			continue;
		}
		if((((strcmp(mp.mnt_fstype,"ufs"))==0) || (strcmp(mp.mnt_fstype,"tmpfs")) ==0)){
			if((sd_ptr=malloc(sizeof(system_disks_t))) == NULL){
				errf("malloc failed (%m)");
				return NULL;
			}
			if((sd_ptr->disk=malloc(sizeof(disk_stat_t))) == NULL){
				errf("malloc failed (%m)");
				return NULL;
			}
			if((sd_ptr->disk->device_name=strdup(mp.mnt_special)) == NULL) {
				errf("strdup failed (%m)");	
				return NULL;
			}
			if((sd_ptr->disk->mnt_point=strdup(mp.mnt_mountp)) == NULL){
				errf("strdup failed (%m)");	
				return NULL;
			}
			sd_ptr->disk->size=((df.f_frsize/1024) * df.f_blocks);
			sd_ptr->disk->used=(((df.f_frsize/1024) * df.f_blocks) -((df.f_frsize/1024) * df.f_bfree));
			sd_ptr->disk->avail=(df.f_frsize/1024) * df.f_bavail;
			sd_ptr->disk->t_inodes=df.f_files;
			sd_ptr->disk->f_inodes=df.f_ffree;
			sd_ptr->next_disk=sd;
			sd=sd_ptr;
		}

	}

	if((fclose(f))!=0){
		errf("Failed to close mnttab");
		return NULL;
	}

#endif
#ifdef LINUX
	if ((f=setmntent("/etc/mtab", "r" ))==NULL){
		errf("Failed to open mounts (%m)");
		return NULL;
	}

	while((mp=getmntent(f))){
		if ((statfs(mp->mnt_dir, &df)) !=0){
			errf("Failed to gets fs stats (%m)");
			continue;
		}

		if((((strcmp(mp->mnt_type, MNTTYPE_NFS))==0) ||  (strcmp(mp->mnt_type,MNTTYPE_IGNORE)) ==0)) continue;
		if((sd_ptr=malloc(sizeof(system_disks_t))) == NULL){
			errf("malloc failed (%m)");
			return NULL;
		}
		if((sd_ptr->disk=malloc(sizeof(disk_stat_t))) == NULL){
			errf("malloc failed (%m)");
			return NULL;
		}
		if((sd_ptr->disk->device_name=strdup(mp->mnt_fsname)) == NULL){	
			errf("strdup failed (%m)");
			return NULL;
		}
		if((sd_ptr->disk->mnt_point=strdup(mp->mnt_dir)) == NULL){
			errf("strdup failed (%m)");
			return NULL;
		}
		sd_ptr->disk->size=((df.f_bsize/1024) * df.f_blocks);
		sd_ptr->disk->used=((df.f_bsize/1024) * df.f_blocks) -((df.f_bsize/1024) * df.f_bfree);
		sd_ptr->disk->avail=((df.f_bsize/1024) * df.f_bavail);
		sd_ptr->disk->t_inodes=df.f_files;
		sd_ptr->disk->f_inodes=df.f_ffree;
		sd_ptr->next_disk=sd;
		sd=sd_ptr;
	}

        endmntent(f);

#endif
#ifdef FREEBSD
	nummnt=getmntinfo(&mp , MNT_LOCAL);
	if (nummnt<=0){
		errf("Failed to get disk stats (%m)");
		return NULL;
	}
	
	for(counter=0;counter<nummnt;counter++){
		if((sd_ptr=malloc(sizeof(system_disks_t))) == NULL){
			errf("malloc failed (%m)");
			return NULL;
		}
		if((sd_ptr->disk=malloc(sizeof(disk_stat_t))) == NULL){
			errf("malloc failed (%m)");
			return NULL;
		}
		if((sd_ptr->disk->device_name=strdup(mp->f_mntfromname))==NULL){
			errf("strdup failed (%m)");
			return NULL;
		}
		if((sd_ptr->disk->mnt_point=strdup(mp->f_mntonname))==NULL){
			errf("strdup failed (%m)");
			return NULL;
		}
		sd_ptr->disk->size=((mp->f_bsize/1024) * mp->f_blocks);
		sd_ptr->disk->used=((mp->f_bsize/1024) * mp->f_blocks) -((mp->f_bsize/1024) * mp->f_bfree);
		sd_ptr->disk->avail=((mp->f_bsize/1024) * mp->f_bavail);
		sd_ptr->disk->t_inodes=mp->f_files;
		sd_ptr->disk->f_inodes=mp->f_ffree;
		sd_ptr->next_disk=sd;
		sd=sd_ptr;
		mp++;
	}
#endif


	sd_ptr=sd;
	xml_disk_stats=strdup("<disk>");
	
	for(counter=0;sd_ptr!=NULL;counter++){
		xml_disk_stats_ptr=xml_disk_stats;
		if((xml_disk_stats=strf("%s<p%d name=\"%s\" mount=\"%s\" kbytes=\"%ld\" used=\"%ld\" avail=\"%ld\" totalinodes=\"%ld\" freeinodes=\"%ld\"></p%d>", xml_disk_stats, counter, sd_ptr->disk->device_name, sd_ptr->disk->mnt_point, sd_ptr->disk->size, sd_ptr->disk->used, sd_ptr->disk->avail, sd_ptr->disk->t_inodes, sd_ptr->disk->f_inodes, counter)) == NULL){
			errf("strf failed (%m)");
			return NULL;
		}
		free(xml_disk_stats_ptr);
		sd_ptr=sd_ptr->next_disk;
	}
	xml_disk_stats_ptr=xml_disk_stats;
	xml_disk_stats=strf("%s</disk>",xml_disk_stats);
	free(xml_disk_stats_ptr);	

	/* Cleaning up */
	sd_ptr=sd;
	while(sd_ptr!=NULL){
		sd=sd_ptr->next_disk;
		free(sd_ptr->disk->device_name);
		free(sd_ptr->disk->mnt_point);
		free(sd_ptr->disk);
		free(sd_ptr);
		sd_ptr=sd;
	}

	return xml_disk_stats;
}

