/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include "ukcprog.h"
#ifdef SOLARIS
#include <sys/stat.h>
#include <sys/swap.h>
#include <unistd.h>
#endif
#ifdef LINUX
#include <string.h>
#endif
#ifdef FREEBSD
#include <sys/types.h>
#include <pwd.h>
#include <unistd.h>
#include <fcntl.h>
#include <kvm.h>
#include <limits.h>
#include <grp.h>
#endif

char *get_swap_stats(){
	char *xml_swap_stats;
	long swaptotal=0;
	long swapfree=0;
	long swapused=0;
#ifdef SOLARIS
	struct anoninfo ai;
	int pagesize;
#endif
#ifdef LINUX
	char *line;
	FILE *f;
#endif
#ifdef FREEBSD
	gid_t gid;
	struct group *kmem_grp;
	struct kvm_swap swapinfo;
	int pagesize;
	kvm_t *kvmd = NULL;
	char errbuf[_POSIX2_LINE_MAX];
#endif

#ifdef SOLARIS
	if((pagesize=sysconf(_SC_PAGESIZE)) == -1){
		errf("failed to get system pagesize (%m)");
		return NULL;
	}
	if (swapctl(SC_AINFO, &ai) == -1) {
		errf("failed to get swap stats (%m)");
		return NULL;
	}
	swaptotal=((ai.ani_max/1024)*pagesize)/1024;
	swapused=((ai.ani_resv/1024)*pagesize)/1024;
	swapfree=swaptotal-swapused;
#endif
#ifdef LINUX
	if ((f=fopen("/proc/meminfo", "r" ))==NULL) {
		errf("Failed to open memory stats (%m)");
		return NULL;
	}
	while((line=fpgetline(f)) != NULL){
		if (((strncmp(line,"Swap: ",6)) == 0)) {
			if((sscanf(line, "%*s %ld %ld %ld", &swaptotal, &swapused, &swapfree)) != 3){
				errf("Failed to read swap details (%m)");
				return NULL;
			}
			break;
		}
	}
	if ((fclose(f)) != 0) {
		errf("Failed to close file (%m)");
		return NULL;
	}
	swaptotal=(swaptotal/1024)/1024;
	swapused=(swapused/1024)/1024;
	swapfree=(swapfree/1024)/1024;
#endif
#ifdef FREEBSD
	/* To do this, this has to be setgid kmem, or be run as root :(
	*/
#define MEM_GROUP "kmem"
	if((kmem_grp=getgrnam(MEM_GROUP))==NULL){
		errf("Failed to find gid for kmem (%m)");
		return NULL;
	}
	gid=getgid();
	if((setegid(kmem_grp->gr_gid))!=0){
		errf("Failed to become gid kmem (%m)\nCan not get swap info.");
		return NULL;
	}
	if((kvmd = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, errbuf)) == NULL){
		errf("Failed to open kvm info to get swap stats (%m)");
		return NULL;
	}
	if ((kvm_getswapinfo(kvmd, &swapinfo, 1,0)) == -1){
		errf("Failed to get swap info (%m)");
		return NULL;
	}
	if((kvm_close(kvmd))!=0){
		errf("Failed to close kvm (%m)");
		return NULL;
	}
	if((setegid(gid))!=0){
		errf("Failed to lose gid kmem (%m)");
		exit(1);
		return NULL;
	}
	pagesize=getpagesize();
	swaptotal=((((long)swapinfo.ksw_total)*pagesize)/1024)/1024;
	swapused=((((long)swapinfo.ksw_used)*pagesize)/1024)/1024;
	swapfree=swaptotal-swapused;
#endif



	if((xml_swap_stats=strf("<swap><total>%ld</total><used>%ld</used><free>%ld</free></swap>", swaptotal, swapused, swapfree)) == NULL){
		errf("strf failed (%m)");
		return NULL;
	}

	return xml_swap_stats;
}
